import 'package:cometchat_chat_uikit/cometchat_chat_uikit.dart';
import 'package:cometchat_chat_uikit/src/ai/ai_conversation_starter/ai_conversation_starter_view.dart';
import 'package:flutter/material.dart';

///[AiConversationStarterDecorator] is a the view model for [AiConversationStarter] it contains all the relevant business logic
///it is also a sub-class of [DataSourceDecorator] which allows any extension to override the default methods provided by [MessagesDataSource]
class AiConversationStarterDecorator extends DataSourceDecorator
    with CometChatUIEventListener {
  late String dateStamp;
  late String _listenerId;
  User? loggedInUser;
  AiConversationStarterConfiguration? configuration;

  AiConversationStarterDecorator(DataSource dataSource, {this.configuration})
      : super(dataSource) {
    dateStamp = DateTime.now().microsecondsSinceEpoch.toString();
    _listenerId = "AiConversationStarter$dateStamp";
    CometChatUIEvents.removeUiListener(_listenerId);
    CometChatUIEvents.addUiListener(_listenerId, this);
    getLoggedInUser();
  }

  getLoggedInUser() async {
    loggedInUser = await CometChat.getLoggedInUser();
  }

  closeCall() {
    CometChat.removeMessageListener(_listenerId);
  }

  @override
  void onConversationChanged(Map<String, dynamic>? id, BaseMessage? lastMessage,
      User? user, Group? group) {
    CometChatTheme theme = configuration?.theme ?? cometChatTheme;
    if (lastMessage == null && id?["parentMessageId"] == null) {
      if (configuration?.onLoad != null) {
        configuration?.onLoad!(
          user,
          group,
          (success, isLoading) {
            if (isLoading) {
              CometChatUIEvents.showPanel(id, CustomUIPosition.composerTop,
                  (context) => _emptyListLoadingIndicator(context, theme));
            } else {
              CometChatUIEvents.showPanel(
                  id, CustomUIPosition.composerTop, (context) => success);
            }
          },
          (error, isError) {
            if (isError) {
              CometChatUIEvents.showPanel(id, CustomUIPosition.composerTop,
                  (context) => _getOnError(context, theme));
            }
          },
        );
      } else {
        onRepliesListBottom(user, group);
      }
    }
  }

  onRepliesListBottom(User? user, Group? group) {
    Map<String, dynamic> id = {};
    String receiverId = "";
    if (user != null) {
      receiverId = user.uid;
    } else if (group != null) {
      receiverId = group.guid;
    }
    id['uid'] = receiverId;
    id['guid'] = receiverId;

    CometChatUIEvents.showPanel(
      id,
      CustomUIPosition.composerTop,
      (context) => AiConversationStarterView(
        style: configuration?.conversationStarterStyle,
        theme: configuration?.theme ?? cometChatTheme,
        user: user,
        group: group,
        emptyStateText: configuration?.emptyStateText,
        errorStateText: configuration?.errorStateText,
        conversationStarterView:
            configuration?.conversationStarterView,
        loadingStateText: configuration?.loadingStateText,
        emptyIconUrl: configuration?.emptyIconUrl,
        loadingStateView: configuration?.loadingStateView,
        loadingIconUrl: configuration?.loadingIconUrl,
        errorStateView: configuration?.errorStateView,
        emptyStateView: configuration?.errorStateView,
        errorIconUrl: configuration?.errorIconUrl,
      ),
    );
  }

  Widget _getOnError(BuildContext context, CometChatTheme _theme) {
   if (configuration?.errorStateView !=
        null) {
      return Center(
        child: configuration?.errorStateView!(context),
      );
    } else {
      return Chip(
        backgroundColor: configuration?.conversationStarterStyle?.backgroundColor ??
            _theme.palette.getBackground(),
        elevation: 4,
        shadowColor:
            configuration?.conversationStarterStyle?.shadowColor ?? _theme.palette.getBackground(),
        label: Row(
          children: [
            Image.asset(
                  configuration?.errorIconUrl ??
                  AssetConstants.repliesError,
              package: UIConstants.packageName,
              color: configuration?.conversationStarterStyle?.errorIconTint ??
                  _theme.palette.getAccent700(),
            ),
            const SizedBox(
              width: 10,
            ),
            Text(
              configuration?.errorStateText ??
                  Translations.of(context).something_went_wrong_error,
              style: configuration?.conversationStarterStyle?.errorTextStyle ??
                  TextStyle(
                    fontSize: _theme.typography.title2.fontSize,
                    fontWeight: _theme.typography.title2.fontWeight,
                    color: _theme.palette.getAccent400(),
                  ),
            ),
          ],
        ),
      );
    }
  }

  Widget _emptyListLoadingIndicator(
      BuildContext context, CometChatTheme _theme) {
    if (configuration?.loadingStateView !=
        null) {
      return SizedBox(
          child: configuration?.loadingStateView!(context));
    } else {
      return Chip(
        backgroundColor: configuration?.conversationStarterStyle?.backgroundColor ??
            _theme.palette.getBackground(),
        elevation: 4,
        shadowColor:
            configuration?.conversationStarterStyle?.shadowColor ?? _theme.palette.getBackground(),
        label: Row(
          children: [
            Image.asset(
                  configuration?.loadingIconUrl ??
                  AssetConstants.spinner,
              package: UIConstants.packageName,
              color: configuration?.conversationStarterStyle?.loadingIconTint ??
                  _theme.palette.getAccent600(),
            ),
            const SizedBox(
              width: 10,
            ),
            Text(
              configuration?.loadingStateText ??
                  Translations.of(context).generating_ice_breakers,
              style: configuration?.conversationStarterStyle?.loadingTextStyle ??
                  TextStyle(
                      fontSize: _theme.typography.subtitle1.fontSize,
                      fontWeight: _theme.typography.subtitle1.fontWeight,
                      color: _theme.palette.getAccent()),
            ),
          ],
        ),
      );
    }
  }

  @override
  String getId() {
    return "conversationStarter";
  }
}
