import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:cometchat_chat_uikit/cometchat_chat_uikit.dart';

///[AiConversationStarterView] is a widget that is rendered as the content view for [AiConversationStarter]
///```dart
/// AiConversationStarterTopView(
///   style: AiConversationStarterStyle(),
///   user: User(),
///   emptyStateText: Text("Error occurred"),
///   loadingStateView: Text("Loading..."),
///   theme: CometChatTheme(),
///   )
/// ```

class AiConversationStarterView extends StatefulWidget {
  const AiConversationStarterView({
    Key? key,
    this.user,
    this.group,
    this.style,
    this.emptyStateText,
    this.errorStateText,
    this.theme,
    this.conversationStarterView,
    this.loadingStateText,
    this.loadingIconUrl,
    this.loadingStateView,
    this.errorIconUrl,
    this.errorStateView,
    this.emptyStateView,
    this.emptyIconUrl,
  }) : super(key: key);

  /// set [User] object, one is mandatory either [user] or [group]
  final User? user;

  ///set [Group] object, one is mandatory either [user] or [group]
  final Group? group;

  ///[style] provides styling to the reply chips/bubbles
  final AiConversationStarterStyle? style;

  ///[theme] sets custom theme
  final CometChatTheme? theme;

  ///[emptyStateText] text to be displayed when the replies are empty
  final String? emptyStateText;

  ///[errorStateText] text to be displayed when error occur
  final String? errorStateText;

  ///[loadingStateText] text to be displayed when loading occur
  final String? loadingStateText;

  ///[conversationStarterView] gives conversation starter view
  final Widget Function(List<String> replies, BuildContext context)?
  conversationStarterView;

  ///[emptyStateView] returns view fow empty state
  final WidgetBuilder? emptyStateView;

  ///[loadingStateView] returns view for loading state
  final WidgetBuilder? loadingStateView;

  ///[errorStateView] returns view for error state
  final WidgetBuilder? errorStateView;

  ///[errorIconUrl] used to set the error icon
  final String? errorIconUrl;

  ///[emptyIconUrl] used to set the empty icon
  final String? emptyIconUrl;

  ///[loadingIconUrl] used to set the loading icon
  final String? loadingIconUrl;


  @override
  State<AiConversationStarterView> createState() =>
      _AiConversationStarterViewState();
}

class _AiConversationStarterViewState
    extends State<AiConversationStarterView> with WidgetsBindingObserver {
  List<String> _replies = [];

  bool isLoading = false;

  bool isError = false;

  bool isKeyboardOpen = false;

  CometChatTheme _theme = cometChatTheme;

  @override
  void initState() {
    getReply();
    super.initState();
    WidgetsBinding.instance.addObserver(this);
  }

  @override
  void dispose() {
    WidgetsBinding.instance.removeObserver(this);
    super.dispose();
  }

  @override
  void didChangeMetrics() {
    final value = WidgetsBinding
        .instance.platformDispatcher.views.first.viewInsets.bottom;
    if (value > 0 != isKeyboardOpen) {
      setState(() {
        isKeyboardOpen = !isKeyboardOpen;
      });
    }
  }

  getReply() async {
    _theme = widget.theme ?? cometChatTheme;
    await getReplies();
  }

  Future<void> getReplies() async {
    setState(() {
      isLoading = true;
    });
    _replies.clear();
    String receiverId = "";
    String receiverType = "";
    if (widget.user != null) {
      receiverId = widget.user!.uid;
      receiverType = CometChatReceiverType.user;
    } else if (widget.group != null) {
      receiverId = widget.group!.guid;
      receiverType = CometChatReceiverType.group;
    }
    await CometChat.getConversationStarter(receiverId, receiverType,
        onSuccess: (reply) {
      _replies = reply;
      setState(() {});
    }, onError: (error) {
     if(kDebugMode) {
       debugPrint("Error in AI conversation starter : ${error.message}");
     }
      isError = true;
      setState(() {});
    });
    isLoading = false;
    setState(() {});
  }

  Widget _getOnError(BuildContext context, CometChatTheme _theme) {
    if (widget.errorStateView != null) {
      return Center(
        child:
            widget.errorStateView!(context),
      );
    } else {
      return Chip(
        backgroundColor:
            widget.style?.backgroundColor ?? _theme.palette.getBackground(),
        elevation: 4,
        shadowColor:
            widget.style?.shadowColor ?? _theme.palette.getBackground(),
        label: Row(
          children: [
            Image.asset(
              widget.errorIconUrl ??
                  AssetConstants.repliesError,
              package: UIConstants.packageName,
              color: widget.style?.errorIconTint ??
                  _theme.palette.getAccent700(),
            ),
            const SizedBox(
              width: 10,
            ),
            Text(
              widget.errorStateText ??
                  Translations.of(context).something_went_wrong_error,
              style: widget.style?.errorTextStyle ??
                  TextStyle(
                    fontSize: _theme.typography.title2.fontSize,
                    fontWeight: _theme.typography.title2.fontWeight,
                    color: _theme.palette.getAccent400(),
                  ),
            ),
          ],
        ),
      );
    }
  }

  Widget _getEmptyView(BuildContext context) {
    if (widget.emptyStateView != null) {
      return widget.emptyStateView!(context);
    } else {
      return Chip(
        backgroundColor:
            widget.style?.backgroundColor ?? _theme.palette.getBackground(),
        elevation: 4,
        shadowColor:
            widget.style?.shadowColor ?? _theme.palette.getBackground(),
        label: Row(
          children: [
            Image.asset(
              widget.emptyIconUrl ??
                  AssetConstants.repliesEmpty,
              package: UIConstants.packageName,
              color: widget.style?.emptyIconTint ??
                  _theme.palette.getAccent600(),
            ),
            const SizedBox(
              width: 10,
            ),
            Text(
              widget.emptyStateText ??
                  Translations.of(context).no_messages_found,
              style: widget.style?.emptyTextStyle ??
                  TextStyle(
                    fontSize: _theme.typography.title1.fontSize,
                    fontWeight: _theme.typography.title1.fontWeight,
                    color: _theme.palette.getAccent400(),
                  ),
            ),
          ],
        ),
      );
    }
  }

  Widget _getLoadingIndicator() {
    if (widget.loadingStateView != null) {
      return widget.loadingStateView!(context);
    } else {
      return Chip(
        backgroundColor:
            widget.style?.backgroundColor ?? _theme.palette.getBackground(),
        elevation: 4,
        shadowColor:
            widget.style?.shadowColor ?? _theme.palette.getBackground(),
        label: Row(
          children: [
            Image.asset(
              widget.loadingIconUrl ??
                  AssetConstants.spinner,
              package: UIConstants.packageName,
              color: widget.style?.loadingIconTint ??
                  _theme.palette.getAccent600(),
            ),
            const SizedBox(
              width: 10,
            ),
            Text(
              widget.loadingStateText ?? Translations.of(context).generating_ice_breakers,
              style: widget.style?.loadingTextStyle ??
                  TextStyle(
                      fontSize: _theme.typography.subtitle1.fontSize,
                      fontWeight: _theme.typography.subtitle1.fontWeight,
                      color: _theme.palette.getAccent()),
            ),
          ],
        ),
      );
    }
  }

  onClickReply(String reply) async {
    Map<String, dynamic> id = {};
    String receiverId = "";
    if (widget.user != null) {
      receiverId = widget.user!.uid;
    } else if (widget.group != null) {
      receiverId = widget.group!.guid;
    }
    id['uid'] = receiverId;
    id['guid'] = receiverId;
    CometChatUIEvents.hidePanel(id, CustomUIPosition.composerTop);

    CometChatUIEvents.ccComposeMessage(reply, MessageEditStatus.inProgress);
  }

  @override
  Widget build(BuildContext context) {
    return (!isKeyboardOpen)
        ? isLoading
            // --- on Loading ---
            ? _getLoadingIndicator()
            :
            // ---on error---
            isError
                ? _getOnError(context, _theme)
                :
                // --- empty replies state ----
                _replies.isEmpty
                    ? _getEmptyView(context)
                    : (widget.conversationStarterView != null)
                        ? widget.conversationStarterView!(
                            _replies, context)
                        : SizedBox(
                            child: SingleChildScrollView(
                              scrollDirection: Axis.vertical,
                              child: Column(
                                crossAxisAlignment: CrossAxisAlignment.start,
                                children: [
                                  ListView.separated(
                                    itemCount: _replies.length,
                                    shrinkWrap: true,
                                    physics:
                                        const NeverScrollableScrollPhysics(),
                                    itemBuilder: (_, int item) {
                                      return GestureDetector(
                                        onTap: () {
                                          onClickReply(_replies[item]);
                                        },
                                        child: Container(
                                          decoration: BoxDecoration(
                                            color: widget.style?.background ??
                                                _theme.palette.getBackground(),
                                            gradient: widget.style?.gradient,
                                            border: widget.style?.border ??
                                                Border.all(
                                                    color: _theme.palette
                                                        .getAccent700(),
                                                    width: 1),
                                            borderRadius: BorderRadius.all(
                                                Radius.circular(widget
                                                        .style?.borderRadius ??
                                                    8)),
                                          ),
                                          child: Padding(
                                            padding: const EdgeInsets.all(8.0),
                                            child: Text(
                                              _replies[item],
                                              style: widget
                                                      .style?.replyTextStyle ??
                                                  TextStyle(
                                                    fontSize: _theme.typography
                                                        .title2.fontSize,
                                                    fontWeight: _theme
                                                        .typography
                                                        .title2
                                                        .fontWeight,
                                                    color: _theme.palette
                                                        .getAccent800(),
                                                  ),
                                            ),
                                          ),
                                        ),
                                      );
                                    },
                                    separatorBuilder: (_, int i) {
                                      return const SizedBox(
                                        height: 5,
                                      );
                                    },
                                  ),
                                ],
                              ),
                            ),
                          )
        : const SizedBox();
  }
}
