import 'package:cometchat_chat_uikit/cometchat_chat_uikit.dart';
import 'package:flutter/material.dart';

///[AiSmartReplyDecorator] is a the view model for [AiSmartReplyExtension] it contains all the relevant business logic
///it is also a sub-class of [DataSourceDecorator] which allows any extension to override the default methods provided by [MessagesDataSource]
class AiSmartReplyDecorator extends DataSourceDecorator
    with CometChatUIEventListener {
  late String dateStamp = "";
  late String _listenerId = "";
  User? loggedInUser;
  AiSmartReplyConfiguration? configuration;

  AiSmartReplyDecorator(DataSource dataSource, {this.configuration})
      : super(dataSource) {
    CometChatUIEvents.removeUiListener(_listenerId);
    dateStamp = DateTime.now().microsecondsSinceEpoch.toString();
    _listenerId = "AiSmartReply$dateStamp";
    CometChatUIEvents.addUiListener(_listenerId, this);
    getLoggedInUser();
  }

  getLoggedInUser() async {
    loggedInUser = await CometChat.getLoggedInUser();
  }

  closeCall() {
    CometChat.removeMessageListener(_listenerId);
  }

  Widget _getOnError(BuildContext context, CometChatTheme _theme) {
   if (configuration?.errorStateView != null) {
      return Center(
        child: configuration?.errorStateView!(context),
      );
    } else {
      return SizedBox(
        height: 270,
        child: Center(
          child: Column(
            mainAxisAlignment: MainAxisAlignment.center,
            mainAxisSize: MainAxisSize.max,
            children: [
              Image.asset(
                    configuration?.errorIconUrl ??
                    AssetConstants.repliesError,
                package: UIConstants.packageName,
                color: configuration?.smartRepliesStyle?.emptyIconTint ??
                    _theme.palette.getAccent700(),
              ),
              Text(
                configuration?.errorStateText ??
                    Translations.of(context).something_went_wrong_error,
                style: configuration?.smartRepliesStyle?.errorTextStyle ??
                    TextStyle(
                      fontSize: _theme.typography.title1.fontSize,
                      fontWeight: _theme.typography.title1.fontWeight,
                      color: _theme.palette.getAccent400(),
                    ),
              ),
            ],
          ),
        ),
      );
    }
  }

  Widget _getLoadingIndicator(BuildContext context, CometChatTheme _theme) {
     if (configuration?.loadingStateView !=
        null) {
      return Center(
          child: configuration?.loadingStateView!(context));
    } else {
      return SizedBox(
        height: 270,
        child: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            Image.asset(
                  configuration?.loadingIconUrl ??
                  AssetConstants.spinner,
              package: UIConstants.packageName,
              color: configuration?.smartRepliesStyle?.loadingIconTint ??
                  _theme.palette.getAccent600(),
            ),
            const SizedBox(
              height: 5,
            ),
            Text(
              configuration?.loadingStateText ??
                  Translations.of(context).generating_replies,
              style: configuration?.smartRepliesStyle?.loadingTextStyle ??
                  TextStyle(
                      fontSize: _theme.typography.subtitle1.fontSize,
                      fontWeight: _theme.typography.subtitle1.fontWeight,
                      color: _theme.palette.getAccent()),
            ),
          ],
        ),
      );
    }
  }

  @override
  void onAiFeatureTapped(User? user, Group? group) {
    CometChatTheme theme = configuration?.theme ?? cometChatTheme;
    Map<String, dynamic> id = {};
    String receiverId = "";
    if (user != null) {
      receiverId = user.uid;
    } else if (group != null) {
      receiverId = group.guid;
    }
    id['uid'] = receiverId;
    id['guid'] = receiverId;
    if (configuration?.onClick != null) {
      configuration?.onClick!(
        user,
        group,
        (success, isLoading, context) {
          if (isLoading) {
            CometChatUIEvents.showPanel(id, CustomUIPosition.composerBottom,
                (context) => _getLoadingIndicator(context, theme));
          } else {
            CometChatUIEvents.showPanel(
                id, CustomUIPosition.composerBottom, (context) => success);
          }
        },
        (error, isError, context) {
          if (isError) {
            CometChatUIEvents.showPanel(id, CustomUIPosition.composerBottom,
                (context) => _getOnError(context, theme));
          }
        },
      );
    } else {
      checkAndShowReplies(user, group);
    }
  }

  checkAndShowReplies(User? user, Group? group) {
    Map<String, dynamic> id = {};

    String receiverId = "";
    if (user != null) {
      receiverId = user.uid;
    } else if (group != null) {
      receiverId = group.guid;
    }
    id['uid'] = receiverId;
    id['guid'] = receiverId;

    CometChatUIEvents.showPanel(
      id,
      CustomUIPosition.composerBottom,
      (context) => AiSmartReplyView(
        style: configuration?.smartRepliesStyle,
        theme: configuration?.theme ?? cometChatTheme,
        user: user,
        group: group,
        emptyStateText: configuration?.emptyStateText,
        errorStateText: configuration?.errorStateText,
        onError: configuration?.onError,
        smartReplyView: configuration?.smartReplyView,
        loadingStateText: configuration?.loadingStateText,
        emptyIconUrl: configuration?.emptyIconUrl,
        loadingStateView: configuration?.loadingStateView,
        loadingIconUrl: configuration?.loadingIconUrl,
        errorStateView: configuration?.errorStateView,
        emptyStateView: configuration?.emptyStateView,
        errorIconUrl: configuration?.errorIconUrl,
      ),
    );
  }

  @override
  String getId() {
    return "aismartreply";
  }
}
