import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:cometchat_chat_uikit/cometchat_chat_uikit.dart';

///[AiSmartReplyView] is a widget that is rendered as the content view for [AiSmartReply]
///
///```dart
/// AiSmartReplyView(
///   style: AiSmartReplyStyle(
///     replyTextStyle: TextStyle(
///       color: Colors.black,
///       fontWeight: FontWeight.bold,
///       fontSize: 14,
///     ),
///   ),
///   theme: CometChatTheme(),
/// );
/// ```
///

class AiSmartReplyView extends StatefulWidget {
  const AiSmartReplyView({
    Key? key,
    this.user,
    this.group,
    this.style,
    this.theme,
    this.onError,
    this.emptyStateText,
    this.errorStateText,
    this.smartReplyView,
    this.loadingStateText,
    this.loadingIconUrl,
    this.loadingStateView,
    this.errorIconUrl,
    this.errorStateView,
    this.emptyStateView,
    this.emptyIconUrl,
  }) : super(key: key);

  ///[style] provides styling to the reply view
  final AiSmartReplyStyle? style;

  ///[theme] sets custom theme
  final CometChatTheme? theme;

  ///[user] user object to show replies
  final User? user;

  ///[group] group object to show replies
  final Group? group;

  ///[emptyStateText] text to be displayed when the replies are empty
  final String? emptyStateText;

  ///[errorStateText] text to be displayed when we get an error
  final String? errorStateText;

  ///[onError] callback triggered in case any error happens when fetching replies
  final OnError? onError;

  ///[smartReplyView] gives smartReply view
  final Widget Function(List<String> replies, BuildContext context)?
      smartReplyView;

  ///[loadingStateText] text to be displayed when loading occur
  final String? loadingStateText;


  ///[emptyStateView] returns view fow empty state
  final WidgetBuilder? emptyStateView;

  ///[loadingStateView] returns view for loading state
  final WidgetBuilder? loadingStateView;

  ///[errorStateView] returns view for error state
  final WidgetBuilder? errorStateView;

  ///[errorIconUrl] used to set the error icon
  final String? errorIconUrl;

  ///[emptyIconUrl] used to set the empty icon
  final String? emptyIconUrl;

  ///[loadingIconUrl] used to set the loading icon
  final String? loadingIconUrl;

  @override
  State<AiSmartReplyView> createState() => _AiSmartReplyViewState();
}

class _AiSmartReplyViewState extends State<AiSmartReplyView> {
  List<String> _replies = [];

  bool isLoading = false;

  bool isError = false;

  late CometChatTheme _theme;

  @override
  void initState() {
    // TODO: implement initState
    getReply();
    super.initState();
  }

  getReply() async {
    _theme = widget.theme ?? cometChatTheme;
    _replies = await getReplies();
  }

  Future<List<String>> getReplies() async {
    setState(() {
      isLoading = true;
    });
    String receiverId = "";
    String receiverType = "";
    if (widget.user != null) {
      receiverId = widget.user!.uid;
      receiverType = CometChatReceiverType.user;
    } else if (widget.group != null) {
      receiverId = widget.group!.guid;
      receiverType = CometChatReceiverType.group;
    }
    Map<String, String> smartReplies = {};
    List<String> aiReplies = [];
    await CometChat.getSmartReplies(receiverId, receiverType,
        onSuccess: (reply) {
      smartReplies = reply;
      if (smartReplies.containsKey("negative")) {
        aiReplies.add(smartReplies["negative"] ?? "");
      }
      if (smartReplies.containsKey("positive")) {
        aiReplies.add(smartReplies["positive"] ?? "");
      }
      if (smartReplies.containsKey("neutral")) {
        aiReplies.add(smartReplies["neutral"] ?? "");
      }
    }, onError: (error) {
      if(kDebugMode) {
        debugPrint("Error in AI smart replies : ${error.message}");
      }
      setState(() {
        isError = true;
      });
    });
    setState(() {
      isLoading = false;
    });
    return aiReplies;
  }

  Widget _getOnError(BuildContext context, CometChatTheme _theme) {
    if (widget.errorStateView != null) {
      return Center(
        child: widget.errorStateView!(context),
      );
    } else {
      return Center(
        child: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          mainAxisSize: MainAxisSize.max,
          children: [
            Image.asset(
              widget.errorIconUrl ??
                  AssetConstants.repliesError,
              package: UIConstants.packageName,
              color:
                  widget.style?.emptyIconTint ?? _theme.palette.getAccent700(),
            ),
            Text(
              widget.errorStateText ??
                  Translations.of(context).something_went_wrong_error,
              style: widget.style?.errorTextStyle ??
                  TextStyle(
                    fontSize: _theme.typography.title1.fontSize,
                    fontWeight: _theme.typography.title1.fontWeight,
                    color: _theme.palette.getAccent400(),
                  ),
            ),
          ],
        ),
      );
    }
  }

  Widget _getEmptyView(BuildContext context) {
    if (widget.emptyStateView != null) {
      return Center(
          child: widget.emptyStateView!(context));
    } else {
      return Column(
        mainAxisAlignment: MainAxisAlignment.center,
        mainAxisSize: MainAxisSize.max,
        children: [
          Image.asset(
            widget.emptyIconUrl ??
                AssetConstants.repliesEmpty,
            package: UIConstants.packageName,
            color: widget.style?.emptyIconTint ?? _theme.palette.getAccent700(),
          ),
          Center(
            child: Text(
              widget.emptyStateText ??
                  Translations.of(context).no_messages_found,
              style: widget.style?.emptyTextStyle ??
                  TextStyle(
                    fontSize: _theme.typography.title1.fontSize,
                    fontWeight: _theme.typography.title1.fontWeight,
                    color: _theme.palette.getAccent400(),
                  ),
            ),
          ),
        ],
      );
    }
  }

  Widget _getLoadingIndicator() {
    if (widget.loadingStateView != null) {
      return Center(
          child: widget.loadingStateView!(context));
    } else {
      return Column(
        mainAxisAlignment: MainAxisAlignment.center,
        children: [
          Image.asset(
            widget.loadingIconUrl ??
                AssetConstants.spinner,
            package: UIConstants.packageName,
            color: widget.style?.loadingIconTint ?? _theme.palette.getAccent600(),
          ),
          const SizedBox(
            height: 5,
          ),
          Text(
            widget.loadingStateText ?? Translations.of(context).generating_replies,
            style: widget.style?.loadingTextStyle ??
                TextStyle(
                    fontSize: _theme.typography.subtitle1.fontSize,
                    fontWeight: _theme.typography.subtitle1.fontWeight,
                    color: _theme.palette.getAccent()),
          ),
        ],
      );
    }
  }

  onClickReply(String reply) async {
    Map<String, dynamic> id = {};
    String receiverId = "";
    if (widget.user != null) {
      receiverId = widget.user!.uid;
    } else if (widget.group != null) {
      receiverId = widget.group!.guid;
    }
    id['uid'] = receiverId;
    id['guid'] = receiverId;
    CometChatUIEvents.hidePanel(id, CustomUIPosition.composerBottom);

    CometChatUIEvents.ccComposeMessage(reply, MessageEditStatus.inProgress);
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      height: 180,
      width: double.maxFinite,
      color: widget.style?.backgroundColor ?? _theme.palette.getBackground(),
      child: isLoading
          // --- on Loading ---
          ? _getLoadingIndicator()
          :
          // ---on error---
          isError
              ? _getOnError(context, _theme)
              :
              // --- empty replies state ----
              _replies.isEmpty
                  ? _getEmptyView(context)
                  // ---- View of smart replies ------
                  : (widget.smartReplyView != null)
                      ? widget.smartReplyView!(_replies, context)
                      : SingleChildScrollView(
                          child: Column(
                            mainAxisSize: MainAxisSize.min,
                            mainAxisAlignment: MainAxisAlignment.center,
                            children: [
                              ListView.separated(
                                itemCount: _replies.length,
                                shrinkWrap: true,
                                physics: const NeverScrollableScrollPhysics(),
                                itemBuilder: (_, int item) {
                                  return Padding(
                                    padding: const EdgeInsets.only(
                                        top: 20, left: 8, right: 8, bottom: 8),
                                    child: GestureDetector(
                                      onTap: () {
                                        onClickReply(_replies[item]);
                                      },
                                      child: Text(
                                        _replies[item],
                                        style: widget.style?.replyTextStyle ??
                                            TextStyle(
                                              fontSize: _theme
                                                  .typography.title2.fontSize,
                                              fontWeight: _theme
                                                  .typography.title2.fontWeight,
                                              color:
                                                  _theme.palette.getAccent800(),
                                            ),
                                      ),
                                    ),
                                  );
                                },
                                separatorBuilder: (_, int i) {
                                  return Divider(
                                    color: widget.style?.dividerTint ??
                                        _theme.palette.getAccent500(),
                                  );
                                },
                              ),
                            ],
                          ),
                        ),
    );
  }
}
